#!/bin/bash
# Version: 2014-09-19-1
#
# unterstützt jetzt name-based virtual Web-Server (mit openssl client mit SNI)
# testet nun auch das Einrichtungs-CA-Zertifikat

export LC_ALL=C

SERVER="$1"
PORT="$2"

JUL2014=0
FEB2014=0
DEZ2006=0
CA=0
SHA1SERVERCERT=0
SHA1CACERT=0

RC=0
CHAIN_HINWEIS_INSTALL=0
CERT_HINWEIS_REPLACE=0

if [ -z "${PORT}" ] ; then
   PORT=443
fi  

type wget >/dev/null 2>&1
if [ $? -ne 0 ] ; then
  echo "Dieses Skript funktioniert nur mit installiertem wget."
  exit 1
fi

type openssl >/dev/null 2>&1
if [ $? -ne 0 ] ; then
  echo "Dieses Skript funktioniert nur mit installiertem openssl."
  exit 1
fi


openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}" -showcerts  </dev/null 2>&1 | grep -q -- "-----BEGIN CERTIFICATE-----"

if [ $? -ne 0 ] ; then
   echo
   echo "Server ${SERVER}, Port ${PORT} ist nicht erreichbar (Tippfehler?)"
   echo
   exit 1
fi   

# Daten des Server-Zertifikats:
SERVERCERT=`openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}"  </dev/null 2>&1 | openssl x509 -text -noout`

SERIAL=`echo "${SERVERCERT}" | egrep '^ *Serial Number *\:' | sed 's/ *Serial Number: *//'`
ISSUER=`echo "${SERVERCERT}" | egrep '^ *Issuer *\:' | sed 's/ *Issuer: *//'`
SUBJECT=`echo "${SERVERCERT}" | egrep '^ *Subject *\:' | sed 's/ *Subject: *//'`
VALIDITY_NOTBEFORE=`echo "${SERVERCERT}" | egrep '^ *Not Before *\:' | sed 's/ *Not Before: *//'`
VALIDITY_NOTAFTER=`echo "${SERVERCERT}" | egrep '^ *Not After *\:' | sed 's/ *Not After *: *//'`
SANS=`echo "${SERVERCERT}" | egrep -1 '^ *X509v3 Subject Alternative Name *\:' | tail -1 | sed 's/^ *//'`
AIA=`echo "${SERVERCERT}" |egrep ' *CA Issuers - URI *\: *http\://cdp1?.pca.dfn.de/'|sed 's/ *CA Issuers - URI://'`
AIAPEM=`echo "${AIA}" | sed 's/\.crt *$/\.pem/'`
HOST=`host "${SERVER}" | grep -v 'mail is handled by' | sed 's/^/        /'`
CHAIN=`echo ${AIA} | sed 's/http:\/\/cdp1/https:\/\/pki/' | sed 's/\/pub\/.*/\/cgi-bin\/pub\/pki?cmd=getStaticPage;name=index;id=2/'`

echo "=========================================================="
echo "Informationen:"
echo "--------------"
echo "Server: ${SERVER}"
echo "${HOST}"
echo "Port: ${PORT}" 
echo "Serverzertifikat:"
echo "  SubjectDN:    ${SUBJECT}"
echo "  Seriennummer: ${SERIAL}"
echo "  IssuerDN:     ${ISSUER}"
echo "  Gültigkeit:   von ${VALIDITY_NOTBEFORE}"
echo "                bis ${VALIDITY_NOTAFTER}"
echo "  Alternative Namen:         ${SANS}"
echo ""
echo "=========================================================="

if [ -z "${AIA}" ] ; then
   echo
   echo "Kein Server-Zertifikat der DFN-PKI!"
   echo
   exit 1
fi

# Test auf das DFN-PCA-Zertifikat vom Februar 2014
# ^LWEV... ist die letzte Zeile des Base64-Codes des Zertifikats. Darin ist
# die Signatur enthalten, ein false positive ist so gut wie ausgeschlossen.
openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}" -showcerts </dev/null 2>&1 | grep -q "^LWEVuniWh7WSz+FQkZ5ZOhXsnlrGhvwoPma61Nk1r608SSKFh+5vjw==$"
if [ $? -eq 0 ] ; then
   FEB2014=1
fi;

# Test auf das DFN-PCA-Zertifikat vom Dezember 2006
# ^qAD... ist die letzte Zeile.
openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}" -showcerts </dev/null 2>&1 | grep -q "^qAD38hw=$"
if [ $? -eq 0 ] ; then
   DEZ2006=1
fi

# Test auf das DFN-PCA-Zertifikat vom Juli 2014
# ^fbV7pQ... ist die letzte Zeile.
openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}" -showcerts </dev/null 2>&1 | grep -q "^fbV7pQLxJMUkYxE0zFqTICp5iDolQpCpZTt8htMSFSMp/CzazDlbVBc=$"
if [ $? -eq 0 ] ; then
   JUL2014=1
fi


# Analyse des Signaturalgorithmus des Serverzertifikats:
openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}"  </dev/null 2>&1 | openssl x509 -text -noout | grep "Signature Algo" | head -1 | grep -q "sha1WithRSAEncryption"
if [ $? -eq 0 ] ; then
   SHA1SERVERCERT=1
fi

# Hole das im AIA des Server-Zertifikats referenzierte Einrichtungs-CA-Zertifikat im DER Format
CACERT=`wget -O - "${AIA}" 2>/dev/null | openssl x509  -inform der 2>/dev/null`

if [ -z "${CACERT}" ] ; then

   CACERT=`wget -O - "${AIA}" 2>/dev/null | openssl x509  2>/dev/null`

   if [ -z "${CACERT}" ] ; then
      echo
      echo "Kann das Einrichtungs-CA-Zertifikat unter ${AIA} nicht per wget herunterladen!"
      echo
      exit 1
   fi
fi

# Bestimmung des DFN-PKI-Sicherheitsniveaus des Einrichtungs-CA-Zertifikats:
echo "${CACERT}" | openssl x509 -text -noout | egrep -q "^ *Issuer: *C=DE, O=DFN-Verein, OU=DFN-PKI, CN=DFN-Verein PCA Global - G01$"
if [ $? -ne 0 ] ; then
   echo
   echo "Das Serverzertifikat ist nicht unter der DFN-PKI Global Hierarchie ausgestellt."
   echo "Es ist nichts zu tun, da das Serverzertifikat aus einer PKI-Hierarchie kommt,"
   echo "die nicht standardmäßig in Browser integriert ist."
   echo
   exit 1
fi

# Analyse des Signaturalgorithmus des Einrichtungs-CA-Zertifikats:
echo "${CACERT}" | openssl x509 -text -noout | grep "Signature Algo" | head -1 | grep -q "sha256WithRSAEncryption"
if [ $? -ne 0 ] ; then
   echo
   echo "Das unter dem AIA des Serverzertifikats ${AIA}"
   echo "referenzierte Einrichtungs-CA-Zertifikat ist nicht mit SHA-256 signiert."
   echo "Bitte wenden Sie sich an die DFN-PCA dfnpca@dfn-cert.de"
   echo
   exit 1
fi



# Extrahiere ID-String des Einrichtungs-CA-Zertifikats
CAID=`echo "${CACERT}" | grep -B1 -e '-----END CERTIFICATE-----' | head -1`


# Test auf das Einrichtungs-CA-Zertifikat in der Kette
openssl s_client -connect "${SERVER}:${PORT}" -servername "${SERVER}" -showcerts </dev/null 2>&1 | grep -q "^${CAID}$"
if [ $? -eq 0 ] ; then
   CA=1
fi

echo "Prüfe das Einrichtungs-CA-Zertifikat:"
echo ""

if [ $CA -eq 0 ] ; then
   echo "    !!!!Not OK: ${SERVER}:${PORT} liefert kein SHA-2 signiertes Einrichtungs-CA-Zertifikat mit der Zertifikatkette aus."
   echo ""
   RC=1
   CHAIN_HINWEIS_INSTALL=1
else
   echo "    OK: ${SERVER}:${PORT} liefert das mit SHA-2 signierte Einrichtungs-CA-Zertifikat aus." 
   echo ""
fi

echo "Prüfe das \"DFN-Verein PCA Global - G01\" Zertifikat:"
echo ""

if [ $FEB2014 -eq 1 ] ; then
   echo "    !!!!Not OK: ${SERVER}:${PORT} liefert das ungültige PCA-Zertifikat vom Februar 2014 aus."
   echo ""
   RC=1
   CHAIN_HINWEIS_INSTALL=1
fi

if [ $DEZ2006 -eq 1 ] ; then
   echo "    !!!!Not OK: ${SERVER}:${PORT} liefert das mit SHA-1 signierte PCA-Zertifikat von Dezember 2006 aus." 
   echo ""
   RC=1
   CHAIN_HINWEIS_INSTALL=1
fi;

if [ $JUL2014 -eq 1 ] ; then
   echo "    OK: ${SERVER}:${PORT} liefert das mit SHA-2 signierte PCA-Zertifikat von Juli 2014 aus." 
   echo ""
fi;

NR=`expr $JUL2014 + $FEB2014 + $DEZ2006`
if [ $NR -gt 1 ] ; then
   echo "    !!!!Not OK: ${SERVER}:${PORT} liefert mehrere, widersprüchliche PCA-Zertifikate aus."
   echo ""
   CHAIN_HINWEIS_INSTALL=1
   RC=1
fi

if [ $NR -eq 0 ] ; then
   echo "    Not OK: ${SERVER}:${PORT} liefert kein PCA-Zertifikat aus."
   echo ""
   CHAIN_HINWEIS_INSTALL=1
   RC=1
fi

echo "Prüfe das Serverzertifikat:"
echo ""

if [ $SHA1SERVERCERT -eq 1 ] ; then
   echo "    !!!!Not OK: Das Serverzertifikat von ${SERVER}:${PORT} ist noch mit SHA-1 signiert,"
   echo "                und wird in Google Chrome zu Warnmeldungen führen."
   echo ""
   CERT_HINWEIS_REPLACE=1
   RC=1
else
   echo "    OK: Das Serverzertifikat von ${SERVER}:${PORT} ist mit SHA-2 signiert."
   echo ""
fi

echo "=========================================================="
echo "Ergebnisse:"

if [ $RC -eq 0 ] ; then
   echo
   echo "Alles OK: ${SERVER}:${PORT} hat eine korrekte SHA-2 Konfiguration von Serverzertifikat und Zertifikatkette. Es ist nichts weiter zu tun."
   echo
else
   echo
   echo "${SERVER}:${PORT} hat keine korrekte SHA-2 Konfiguration von Serverzertifikat und Zertifikatkette."
   echo

   if [ $CHAIN_HINWEIS_INSTALL -eq 1 ] ; then
     echo "*Wichtig*: Bitte installieren bzw. ersetzen Sie im Server die neue SHA-2 Zertifikatkette."
     echo "           Die Zertifikatkette erhalten Sie über:"
     echo "           ${CHAIN}"
     echo ""
   fi  


   if [ $CERT_HINWEIS_REPLACE -eq 1 ] ; then
     echo "*Wichtig*: Bitte tauschen Sie das Serverzertifikat gegen ein neues mit SHA-2-Signatur aus."
     echo ""
   else
     echo "Das Serverzertifikat selbst müssen Sie *nicht* austauschen."
   fi  
fi

exit $RC
